#pragma once

/* Copyright (C) 2014-2016 Achim Turan, mail@achim-turan.de
 * https://github.com/AchimTuran/asplib
 *
 * This file is part of asplib (Achim's Signal Processing LIBrary)
 *
 * asplib (Achim's Signal Processing LIBrary) is free software:
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 *
 * asplib (Achim's Signal Processing LIBrary) is distributed
 * in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with asplib (Achim's Signal Processing LIBrary).
 * ifnot, see <http://www.gnu.org/licenses/>.
 *
 */



#include "Core/os/asplib_os.h"
#include "Core/Constants_Typedefs/asplib_Typedefs.h"

#include "Core/ComplexVector/asplib_TComplexVector.hpp"
#include "Core/Constants_Typedefs/asplib_DataFmtDefines.hpp"

#include <pmmintrin.h>


namespace asplib
{
class CComplexVector_SSE3Float;

template <>
struct complex_vector_type<CComplexVector_SSE3Float>
{
  typedef float complex_type;
};


class CComplexVector_SSE3Float : public TComplexVector<CComplexVector_SSE3Float>
{
public:
  inline CComplexVector_SSE3Float() : m_Value(_mm_setr_ps(0.0f, 0.0f, 0.0f, 0.0f)) {}
  inline CComplexVector_SSE3Float(const float& R1, const float& R2) : m_Value(_mm_setr_ps(R1, 0.0f, R2, 0.0f)) {}
  inline CComplexVector_SSE3Float(const float& R1, const float& I1, const float& R2, const float& I2) : m_Value(_mm_setr_ps(R1, I1, R2, I2)) {}
  inline CComplexVector_SSE3Float(const __m128& rhs) : m_Value(rhs) {}

  inline void Scale(const float& Val) 
  {
    __m128 _scale = _mm_load1_ps(&Val);
    m_Value = _mm_mul_ps(m_Value, _scale);
  }

  inline CComplexVector_SSE3Float& operator=(const __m128& rhs)
  {
    m_Value = rhs;
    return *this;
  }

  static const uint32_t    vectorSize;
  static const asplibFmt_t typeID;

  inline operator __m128() const { return m_Value; }

private:
  __m128 m_Value;
};

inline CComplexVector_SSE3Float operator+(const CComplexVector_SSE3Float& LHS, const CComplexVector_SSE3Float& RHS)
{
  //Z0[ii][0] += Zi[ii][0];
  //Z0[ii][1] += Zi[ii][1];
  //__m128 _Zi = _mm_load_ps(&Zi[ii]);
  //__m128 _Z0 = _mm_load_ps(&Z0[ii]);
  //_mm_store_ps(&Z0[ii], _mm_add_ps(_Z0, _Zi));

  //__m128 _Zi = _mm_load_ps(&Zi[ii]);
  //__m128 _Z0 = _mm_load_ps(&Z0[ii]);
  return _mm_add_ps(LHS, RHS);
}

inline CComplexVector_SSE3Float operator-(const CComplexVector_SSE3Float& LHS, const CComplexVector_SSE3Float& RHS)
{
  return _mm_sub_ps(LHS, RHS);
}

inline CComplexVector_SSE3Float operator*(const CComplexVector_SSE3Float& LHS, const CComplexVector_SSE3Float& RHS)
{
  // add/subtract, scale and store operations
  // duplicate values
  // _A1 = _mm_moveldup_ps: src{ a1, b1, a2, b2 } --> val2 { a1, a1, a2, a2 }
  // _A2 = _mm_movehdup_ps: src{ a1, b1, a2, b2 } --> val3 { b1, b1, b2, b2 }
  // a = calc { a1*c1, a1*d1, a2*c2, a2*d2 } --> sse3 multiply
  // b = reorder im and re numbers { c1, d1, c2, d2 } --> { d1, c1, d2, c2 } and multiply { b1*d1, b1*c1, b2*d2, b2*c2 }
  // A = _mm_addsub_ps: ret { a1*c1 - b1*d1, j(a1*d1 + b1*c1), a2*c2 - b2*d2, j(a2*d2 + b2*c2) }
  // _mm_store_ps: C[0] = result0, C[1] = result1, C[2] = result2, C[3] = result3
  return _mm_addsub_ps(_mm_mul_ps(_mm_moveldup_ps(LHS), RHS), _mm_mul_ps(_mm_movehdup_ps(LHS), _mm_shuffle_ps(RHS, RHS, _MM_SHUFFLE(2, 3, 0, 1))));
}

inline CComplexVector_SSE3Float operator*(const CComplexVector_SSE3Float& LHS, const float& Val)
{
  __m128 _scale = _mm_load1_ps(&Val);
  return _mm_mul_ps(LHS, _scale);
}

inline CComplexVector_SSE3Float operator/(const CComplexVector_SSE3Float& LHS, const CComplexVector_SSE3Float& RHS)
{
  // todo
  return CComplexVector_SSE3Float();
}
}
